@@ -3,6 +3,10 @@
  * Copyright (c) 1997-99  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * $Log: hotkey.c,v $
+ * Revision 2.2  2000/01/15 10:34:57  jab
+ * changed Fn detection to new /proc interface
+ * fixed a whole slew of bugs so it actually works
+ *
  * Revision 2.1  1999/08/09 20:25:24  jab
  * use the SCI to get the settings rather than the CMOS
  * added support for new HCI based Fn detection
@@ -37,23 +41,23 @@
  * Initial revision
  *
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  * 
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software 
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 
-static char const rcsid[]="$Id: hotkey.c,v 2.1 1999/08/09 20:25:24 jab Exp jab $";
+static char const rcsid[]="$Id: hotkey.c,v 2.2 2000/01/15 10:34:57 jab Exp jab $";
 
 
 #include<stdio.h>
@@ -78,17 +82,16 @@ static char const rcsid[]="$Id: hotkey.c,v 2.1 1999/08/09 20:25:24 jab Exp jab $
 
 
 #include"hotkey.h"
+#include"toshiba.h"
 #include"sci.h"
-#include"hci.h"
 
 
 static int popup = 0;
-static int test = 0;
+static int type = MAUI;
 static int language = ENGLISH;
 static int oldpower = -1;
 static int oldbattery = -1;
 static int oldvolume = -1;
-static int oldevent = 0x0100;
 
 static Display *display = NULL;
 static Screen *scrptr;
@@ -100,6 +103,7 @@ static XFontSet font;
 static unsigned long pixels[14];
 static int depth = 0;
 
+
 #define FONT "-adobe-helvetica-bold-r-normal-*-12-*-*-*-*-*-*-*,\
 -*-*-*-r-normal--14-*-*-*-*-*"
 
@@ -120,10 +124,39 @@ speaker volume a Toshiba laptop using the Fn key combinations\n\n\
 Report bugs to jonathan@buzzard.org.uk\n\
 "
 
+/*
+ * Determine the model type for the different battery save settings
+ */
+int GetModelType(void)
+{
+	SMMRegisters reg;
+	int model;
+
+	SciOpenInterface();
+	reg.ebx = SCI_BATTERY_SAVE;
+	SciGet(&reg);
+	SciCloseInterface();
+	
+	switch (reg.edx & 0xffff) {
+		case 2:
+			model = MAUI;
+			break;
+		case 3:
+			model = STOCKHOLM;
+			break;
+		default:
+			model = MAUI;
+			break;
+	}
+
+	return model;
+}
+
+
 /*
  * Create a window 440 pixels wide by 80 pixels high in the centre of the screen
  */
-void CreateBox()
+void CreateBox(void)
 {
 	XSetWindowAttributes xattributes;
 	int x,y;
@@ -162,7 +195,7 @@ void CreateBox()
 /*
  * Get rid of the popup window
  */
-void DestroyBox()
+void DestroyBox(void)
 {
 	XUnmapWindow(display, box);
 	XDestroyWindow(display, box);
@@ -175,7 +208,7 @@ void DestroyBox()
 
 
 /*
- * Paint the popup windows main elements with either resume or power icons
+ * Paint the popup windows main elements with resume, power or volume icons
  */
 void PaintBox(int style)
 {
@@ -308,15 +341,20 @@ void UpdateStatus(int key)
 	SciOpenInterface();
 	reg.ebx = SCI_BATTERY_SAVE;
 	SciGet(&reg);
-	battery = reg.ecx;
+	battery = (int) ((reg.edx & 0xffff)-(reg.ecx & 0xffff));
 	reg.ebx = SCI_POWER_UP;
 	SciGet(&reg);
-	power = reg.ecx;
+	power = (int) (reg.ecx & 0xffff);
 	reg.ebx = SCI_SPEAKER_VOLUME;
 	SciGet(&reg);
-	volume = reg.ecx;
+	volume = (int) (reg.ecx & 0xffff);
 	SciCloseInterface();
 
+	if (battery<0)
+		battery=2;
+	if (power<0)
+		power=1;
+
 	/* exit if the mode has not changed */
 
 	if ((battery==oldbattery) && (key==0xbc))
@@ -332,23 +370,33 @@ void UpdateStatus(int key)
 	if (key==0x3c) {
 		switch (battery) {
 			case 0:
+				SelectionBox(0);
+				index = 2;
+				break;
+			case 1:
 				SelectionBox(1);
+				index = (type==STOCKHOLM) ? 1 : 0;			
 				break;
 			case 2:
-				SelectionBox(0);
+				SelectionBox(2);
+				index = (type==STOCKHOLM) ? 0 : 3;
 				break;
 			case 3:
-				SelectionBox(2);
+				SelectionBox(3);
+				index = 3;
 				break;
 			default:
-				SelectionBox(5); /* this leaves no selection boxes */
+				/* this leaves no selection boxes */
+				index = 0;
+				SelectionBox(5); 
 				break;
 		}
 		XSetForeground(display, gc, pixels[SILVER]);
 		XFillRectangle(display, box, gc, 258, 32, 166, 15);
 		XSetForeground(display, gc, pixels[BLACK]);
 		XmbDrawString(display, box, font, gc, 259, 43,
-			(save[language][battery]), strlen(save[language][battery]));
+			(save[language][index]),
+				strlen(save[language][index]));
 	}
 
 	if (key==0x3d) {
@@ -361,33 +409,13 @@ void UpdateStatus(int key)
 	}
 
 	if (key==0x3e) {
-		switch (volume) {
-			case 0:
-				SelectionBox(0);
-				index = 0;
-				break;
-			case 1:
-				SelectionBox(1);
-				index = 1;
-				break;
-			case 4:
-				SelectionBox(2);
-				index = 2;
-				break;
-			case 7:
-				SelectionBox(3);
-				index = 3;
-				break;
-			default:
-				SelectionBox(5); /* this leaves no selection boxes */
-				index = 0;
-				break;
-		}
+		SelectionBox(volume);
 		XSetForeground(display, gc, pixels[SILVER]);
 		XFillRectangle(display, box, gc, 258, 32, 166, 15);
 		XSetForeground(display, gc, pixels[BLACK]);
 		XmbDrawString(display, box, font, gc, 259, 43,
-			(speaker[language][index]), strlen(speaker[language][index]));
+			(speaker[language][volume]),
+				strlen(speaker[language][volume]));
 	}
 
 	XFlush(display);
@@ -407,21 +435,8 @@ void UpdateStatus(int key)
  */
 void catch(int x)
 {
-	SMMRegisters reg;
-
 	fprintf(stderr, "hotkey: caught signal %d, cleaning up...\n", x);
 
-	/* disable reporting event reporting */
-
-	reg.eax = HCI_SET;
-	reg.ebx = HCI_SYSTEM_EVENT;
-	reg.ecx = HCI_DISABLE;
-	HciFunction(&reg);
-	reg.eax = HCI_SET;
-	reg.ebx = HCI_HOTKEY_EVENT;
-	reg.ecx = HCI_DISABLE;
-	HciFunction(&reg);
-
 	if (popup!=0)
 		DestroyBox();
 
@@ -440,13 +455,12 @@ void catch(int x)
 
 int main(int argc, char *argv[])
 {
-	SMMRegisters reg;
 	int count,key,loop,refresh,pid,version;
 	long mask;
 	FILE *str;
 	XEvent event;
 	XColor color;
-	char **list,*def_string,*locale;
+	char **list,*def_string,*locale,buffer[64];
 
 
 	/* do some quick checks on the laptop */
@@ -466,13 +480,13 @@ int main(int argc, char *argv[])
 			/* test to see if the other hotkey is still running */
 
 			if (kill(pid, SIGUSR1)==0) {
-				fprintf(stderr, "hotkey: Already running as process %d.\n",
-					pid);
+				fprintf(stderr, "hotkey: Already running as "
+					"process %d.\n", pid);
 				return 1;
 			}
 
-			fprintf(stderr, "hotkey: process %d appears to have died,"
-				" continuing\n", pid);
+			fprintf(stderr, "hotkey: process %d appears to have "
+				"died, continuing\n", pid);
 			unlink(PID_FILE);
 		}
 	}
@@ -520,10 +534,6 @@ int main(int argc, char *argv[])
 			argv++;
 			depth = 1;
 			}
-		if (!strcmp(argv[0], "-t") || !strcmp(argv[0], "--test")) {
-			argv++;
-			test = 1;
-			}
 		if (!strcmp(argv[0], "-en")) {
 			language = ENGLISH;
 			locale = "C";
@@ -595,8 +605,9 @@ int main(int argc, char *argv[])
 			color.green = rgb[loop][1];
 			color.blue = rgb[loop][2];		
 			if (!XAllocColor(display, cmap, &color)) {
-				fprintf(stderr, "hotkey: XAllocColor failed on color %d,"
-					"reverting to monochrome.\n", loop);
+				fprintf(stderr, "hotkey: XAllocColor failed on "
+					"color %d, reverting to monochrome.\n",
+					loop);
 				XFreeColors(display, cmap, pixels, loop, 0);
 				depth = 1;
 				break;
@@ -627,7 +638,7 @@ int main(int argc, char *argv[])
 
 	font = XCreateFontSet(display, FONT, &list, &count, &def_string);
 
-	/* display warning if there are charsets for which no fonts can be found */
+	/* display warning if there are charsets for which fonts can be found */
 
 	if (count>0) {
 		fprintf(stderr, "hotkey: The following charsets are missing:\n");
@@ -663,37 +674,9 @@ int main(int argc, char *argv[])
 
 	popup = 0;
 	refresh = 0;
+	type = GetModelType();
 
-	/* only use the new method when emulated on older laptops unless
-	   specified on the command line. If APM support exists in the
-	   kernel this will currently causes the laptop to freeze. */
-
-	reg.eax = HCI_GET;
-	reg.ebx = HCI_HOTKEY_EVENT;
-	if ((HciFunction(&reg)==HCI_SUCCESS) && (test==0)) {
-
-		fprintf(stderr, "hotkey: use -t option to test the program, "
-			"warning likely to lock machine.\n");
-
-		if (depth>=8)
-			XFreeColors(display, cmap, pixels, 10, 0);
-		XFreeFontSet(display, font);
-		XCloseDisplay(display);
-
-		unlink(PID_FILE);
-		exit(1);
-	}
-
-	/* setup system event reporting */
-
-	reg.eax = HCI_SET;
-	reg.ebx = HCI_SYSTEM_EVENT;
-	reg.ecx = HCI_ENABLE;
-	HciFunction(&reg);
-	reg.eax = HCI_SET;
-	reg.ebx = HCI_HOTKEY_EVENT;
-	reg.ecx = HCI_ENABLE;
-	HciFunction(&reg);
+	/* loop forever */
 
 	for (;;) {
 
@@ -701,16 +684,17 @@ int main(int argc, char *argv[])
 
 		usleep(200000);
 
-		/* read the Fn key status */
+		/* open /proc/toshiba for reading */
 
-		key = 0;
-		reg.eax = HCI_GET;
-		reg.ebx = HCI_SYSTEM_EVENT;
-		reg.ecx = oldevent;
-		if (HciFunction(&reg)==HCI_SUCCESS)
-			if ((reg.ecx & 0x0100)!=0)
-				key = (int) reg.ecx & 0xff;
-		oldevent = reg.ecx;
+		if (!(str = fopen(TOSH_PROC, "r")))
+			return -1;
+
+		/* scan in Fn key information */
+
+		fgets(buffer, sizeof(buffer)-1, str);
+		fclose(str);
+		buffer[sizeof(buffer)-1] = '\0';
+		sscanf(buffer, "%*s %*x %*d.%*d %*d.%*d %*x %x\n", &key);
 
 		/* get rid of status window if Fn key released */
 
@@ -730,8 +714,9 @@ int main(int argc, char *argv[])
 				popup = key & 0x7f;
 				PaintBox(popup);
 			}
-			if ((key & 0x7f)==popup)
+			if ((key & 0x7f)==popup) {
 				UpdateStatus(key);
+			}
 
 			/* event loop handler */
 
