@@ -370,8 +370,6 @@ int GetMenuItemNumber(MenuItem *menu, int setting)
 			return i;
 	}
 
-	printf("GetMenuItemNumber failed\n");
-
 	return -1;
 }
 
@@ -384,7 +382,6 @@ void SetBatteryOptions(int mode)
 	int i = 0;
 	struct BatterySettings settings;
 
-	printf("SET BATTERY OPTIONS CALLED %d %d\n", mode, savetype);
 
 	if (mode==SCI_MAINS) {
 		settings.save = external.save;
@@ -438,7 +435,14 @@ void SetBatteryOptions(int mode)
 	if ((i = GetMenuItemNumber(sleepmenu, settings.sleep))!=-1)
 		gtk_option_menu_set_history(GTK_OPTION_MENU(sleep), i);
 
-	if ((i = GetMenuItemNumber(lcdsupermenu, settings.lcd))!=-1)
+	if (lcdtype==BRIGHT)
+		if ((i = GetMenuItemNumber(lcdbrightmenu, settings.lcd))!=-1)
+			gtk_option_menu_set_history(GTK_OPTION_MENU(lcd), i);
+	else if (lcdtype==SUPER)
+		if ((i = GetMenuItemNumber(lcdsupermenu, settings.lcd))!=-1)
+			gtk_option_menu_set_history(GTK_OPTION_MENU(lcd), i);
+	else
+		if ((i = GetMenuItemNumber(lcdbackmenu, settings.lcd))!=-1)
 			gtk_option_menu_set_history(GTK_OPTION_MENU(lcd), i);
 
 	if ((i = GetMenuItemNumber(coolingmenu, settings.cooling))!=-1)
@@ -453,7 +457,6 @@ void SetBatteryOptions(int mode)
  */
 void SetBatterySensitivity(int save)
 {
-	printf("BATTERY SENSITIVITY = %d\n", save);
 
 	if (save==SCI_USER_SETTINGS) {
 		gtk_widget_set_sensitive(GTK_WIDGET(hdd), TRUE);
@@ -478,18 +481,20 @@ void SetBatterySensitivity(int save)
 
 
 /*
- * Change the option menus the power source is changed by the radio buttons
+ * Change the option menus when the power source is changed by the radio buttons
  */
 void PowerSourceCallback(GtkWidget *widget, int data)
 {
 	if (source!=(int) data) {
+
 	        source = (int) data;
-		SetBatteryOptions(source);
 
 		if (source==SCI_MAINS)
 			SetBatterySensitivity(external.save);
 		else
 			SetBatterySensitivity(internal.save);
+
+		SetBatteryOptions(source);
 	}
 
 	return;
@@ -506,8 +511,8 @@ void BatteryCallback(GtkWidget *widget, int data)
 	else
 		internal.save = data;
 
-	SetBatteryOptions(source);
 	SetBatterySensitivity(data);
+	SetBatteryOptions(source);
 
 	return;
 }
@@ -848,8 +853,8 @@ void MachineCancelCallback(GtkWidget *widget, GtkWidget *dialog)
 	SetSetting(SCI_POWER_UP, machine.powerup, ERROR_GUI);
 	SetSetting(SCI_PANEL_POWER, machine.panelpower, ERROR_GUI);
 
-	printf("FIXME: need to set alarm power to it's entry value "
-		"on canceling the machine settings dialog\n");
+/*	printf("FIXME: need to set alarm power to it's entry value "
+		"on canceling the machine settings dialog\n");*/
 
 	/* exit deleting pid file */
 
@@ -976,9 +981,9 @@ void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 	lcdtype = BACKLIGHT;
 	reg.ebx = SCI_LCD_BRIGHTNESS;
 	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
-		if ((reg.ecx & 0xffff)==0x8001)
+		if ((reg.ebx & 0xffff)==0x8001)
 			lcdtype = SUPER;
-		if ((reg.ecx & 0xffff)==0x0000)
+		if ((reg.ebx & 0xffff)==0x0000)
 			lcdtype = BRIGHT;
 	}
 	
@@ -1296,6 +1301,9 @@ void VolumeUpdate(GtkAdjustment *adjustment, int *junk)
 		case 3:
 			gtk_label_set(GTK_LABEL(level), "100%");
 			break;
+		default:
+			gtk_label_set(GTK_LABEL(level), "100%");
+			break;
 	}
 
 	old = loudness;
@@ -1371,7 +1379,7 @@ void AddSoundPage(GtkWidget *notebook, GtkWidget *window)
 	gtk_box_pack_start(GTK_BOX(speaker), label, FALSE, FALSE, 4);
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 1.0);
 
-	level = gtk_label_new("");
+	level = gtk_label_new("100%");
 	gtk_box_pack_start(GTK_BOX(speaker), level, FALSE, FALSE, 4);
 
 	slider = gtk_adjustment_new(4, 1, 5, 1, 1, 1);
