@@ -1,6 +1,6 @@
 /* thotswap.c -- enable hot swapping of IDE devices in the Selectbay
  *
- * Copyright (c) 1997-99  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
+ * Copyright (c) 1997-2000  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * Based on patches to hdparm by Christian Lademann <cal@zls.de> to use
  * the HDIO_UNREGISTER_HWIF and HDIO_SCAN_HWIF ioctls found in 2.2.x and
@@ -71,6 +71,7 @@ static int removed = 0;
 static int wall = 0;
 static int xmessage = 0;
 static int quite = 0;
+static int console = 1;
 
 
 #define PMAX 5
@@ -85,8 +86,8 @@ static int ptail = 0;
 #define BEEP_ERR  4000
 
 #define VERSION_STRING "\
-Toshiba HotSwap  version 1.0\n\
-Copyright (c) 1999 Jonathan A. Buzzard <jonathan@buzzard.org.uk>\n"
+Toshiba HotSwap  version 1.1\n\
+Copyright (c) 2000 Jonathan A. Buzzard <jonathan@buzzard.org.uk>\n"
 
 #define USAGE \
 "Usage: thotswap [OPTION]\n\n\
@@ -100,12 +101,6 @@ Copyright (c) 1999 Jonathan A. Buzzard <jonathan@buzzard.org.uk>\n"
 Report bugs to jonathan@buzzard.org.uk\n"
 
 
-#define EMERGENCY_ALERT \
-"You have almost no battery\n\
-capacity left on this battery!\n\
-Find an alternative power source\n\
-immediately!"
-
 /*
  * Catch any signals and exit the program nicely
  */
@@ -175,51 +170,44 @@ void ShovelDeadChildren()
 		HandleDeadChild(pid[ptail], retstatus[ptail]);
 		ptail = (ptail==PMAX-1) ? 0 : ++ptail;
 	}
-
 	return;
 }
 
 
 /*
- * Use xmessage to display the battery alert messages
+ * Use xmessage to display information messages
  */
-int DisplayMessage(char *message, char *title)
+int DisplayXMessage(char *message)
 {
-	int pid,save,lead[2];
+	int pid,lead[2];
+
+	if (xmessage==0)
+		return -1;
 
 	/* create the pipe */
 
-	if (pipe(lead)<0) {
+	if (pipe(lead)!=0) {
 		fprintf(stderr, "thotswap: unable to create pipe to "
-			"xmessage\n");
+			"xmessage.\n");
 		return -1;
 	}
 
 	if (fcntl(lead[0], F_SETFL, O_NDELAY)<0) {
-		fprintf(stderr, "thotswap: fatal fcntl call\n");
+		fprintf(stderr, "thotswap: fatal fcntl call.\n");
 		return -1;
 	}
 
 	pid = fork();
 	if (pid>0) {
-		save = dup(1);
-		if (close(1)<0)
-			fprintf(stderr, "PERROR closing stdout\n");
-		if (dup(lead[1])<0)
-			fprintf(stderr, "PERROR duplicating file descriptor\n");
+		write(lead[1], message, strlen(message));
 		close(lead[0]);
 		close(lead[1]);
-		fprintf(stdout, "%s\n", EMERGENCY_ALERT);
-		close(1);
-		dup(save);
 		return 127;
 	} else if (pid==0) {
-		char *argv[9] = {"xmessage", "-buttons", "Ok", "-title", title,
-			"-center", "-file", "-", NULL};
-		if (close(0)<0)
-			fprintf(stderr, "CERROR closing stdin\n");
-		if (dup(lead[0])<0)
-			fprintf(stderr, "CERROR duplicating file descriptor\n");
+		char *argv[9] = {"xmessage", "-buttons", "Ok", "-title",
+			"Toshiba HotSwap", "-center", "-file", "-", NULL};
+		close(0);
+		dup(lead[0]);
 		close(lead[0]);
 		close(lead[1]);
 		execv(XMESSAGE, argv);
@@ -230,10 +218,68 @@ int DisplayMessage(char *message, char *title)
 }
 
 
+/*
+ * Use syslog, wall and stderr as appropriate to display the messages
+ */
+int WriteMessage(char *message)
+{
+	int pid,lead[2];
+
+	/* write message to stderr or log via syslog */
+
+	if (console==0)
+		fprintf(stderr, "%s", message);
+	else
+		syslog(LOG_INFO, message);
+
+	/* only continue if wall messages are required */
+
+	if (wall==0)
+		return 127;
+
+	/* create the pipe */
+
+	if (pipe(lead)!=0) {
+		fprintf(stderr, "thotswap: unable to create pipe to wall.\n");
+		return -1;
+	}
+
+	if (fcntl(lead[0], F_SETFL, O_NDELAY)<0) {
+		fprintf(stderr, "thotswap: fatal fcntl call.\n");
+		return -1;
+	}
+
+	pid = fork();
+	if (pid>0) {
+		write(lead[1], "thotswap: ", 10);
+		write(lead[1], message, strlen(message));
+		close(lead[0]);
+		close(lead[1]);
+		return 127;
+	} else if (pid==0) {
+		char *argv[2] = {"wall", NULL};
+		close(0);
+		dup(lead[0]);
+		close(lead[0]);
+		close(lead[1]);
+		execv(WALL, argv);
+		return 127;
+	}
+
+	return -1;
+}
+
+
+/*
+ * Make the PC speaker produce a beep
+ */
 void Beep(unsigned int ms, unsigned int freq)
 {
 	int fd, arg;
 
+	if (quite==1)
+		return;
+
 	fd = open("/dev/console", O_RDWR);
 	if (fd < 0)
 		return;
@@ -276,7 +322,6 @@ int DeviceMounted(void)
 }
 
 
-
 /*
  * Try and unregister the IDE device in the SelectBay
  */
@@ -316,13 +361,18 @@ int SelectbayUnregister(void)
 	if (err==-1) {
 		Beep(BEEP_TIME, BEEP_ERR);
 		syslog(LOG_INFO, "WARNING: unable to unregister the device in "
-				"the SelectBay, please re-lock");
+				"the SelectBay, please re-lock\n");
+		DisplayXMessage("WARNING: unable to unregister\n        "
+			"the device in the SelectBay.\n         "
+			"Please re-lock");
 	}
 
 	close (fd);
+
 	return err;
 }
 
+
 /*
  * Force a rescan of the IDE bus to find any devices in the SelectBay
  */
@@ -356,11 +406,14 @@ int SelectbayRescan(void)
 
 	if (err==-1) {
 		Beep(BEEP_TIME, BEEP_WARN);
-		syslog(LOG_INFO, "WARNING: error rescanning the IDE bus");
+		WriteMessage("WARNING: error rescanning the IDE bus.\n");
+		DisplayXMessage("WARNING: error rescanning the IDE bus");
 	}
 	
 	Beep(BEEP_TIME, BEEP_OK);
-	close (fd);
+	WriteMessage("device succesfully inserted into SelectBay.");
+	DisplayXMessage("Device succesfully inserted into SelectBay.");
+	close(fd);
 	return err;
 }
 
@@ -381,8 +434,10 @@ void SelectbayUnlocked(void)
 
 	if (DeviceMounted()==1) {
 		Beep(BEEP_TIME, BEEP_ERR);
-		syslog(LOG_INFO, "WARNING: device in SelectBay contains "
-				"mounted filesystems, please re-lock");
+		WriteMessage("WARNING: device in SelectBay contains "
+				"mounted filesystems, please re-lock.\n");
+		DisplayXMessage("WARNING: device in SelectBay contains\n"
+			"         mounted filesystems. Please re-lock.");
 		return;
 	}
 
@@ -407,8 +462,10 @@ void SelectbayUnlocked(void)
 
 		if (reg.ecx==HCI_ENABLE) {
 			Beep(BEEP_TIME, BEEP_ERR);
-			syslog(LOG_INFO, "WARNING: device in SelectBay is boot "
-				"device, please re-lock");
+			WriteMessage("WARNING: device in SelectBay is boot "
+				"device, please re-lock.\n");
+			DisplayXMessage("WARNING: device in SelectBay is boot\n"
+				"         device. Please re-lock.");
 			return;
 		}
 
@@ -416,19 +473,23 @@ void SelectbayUnlocked(void)
 
 		if (SelectbayUnregister()<0) {
 			Beep(BEEP_TIME, BEEP_ERR);
-			syslog(LOG_INFO, "WARNING: unable to remove device in "
-				"the SelectBay, please re-lock");
+			WriteMessage("WARNING: unable to remove device in the "
+				"SelectBay, please re-lock.\n");
+			DisplayXMessage("WARNING:  unable to remove device in\n"
+				"         the SelectBay. Please re-lock.");
 			return;
 		}
 
 		Beep(BEEP_TIME, BEEP_OK);
-		syslog(LOG_INFO, "device in the SelectBay removed");
+		WriteMessage("device in the SelectBay removed.\n");
+		DisplayXMessage("Device in the SelectBay sucessfully removed.");
 		removed = 1;
 	}
 	
 	return;
 }
 
+
 /*
  * The Selectbay has been locked try and rescan IDE bus.
  */
@@ -468,7 +529,7 @@ void Daemon(void)
 	signal(SIGUSR1, SIG_IGN);
 
 	/* parent */
-/*
+
 	if ((pid=fork())) {
 		if ((str=fopen(_PATH_VARRUN PID_FILE, "w"))) {
 			fprintf(str, "%d\n", pid);
@@ -476,9 +537,9 @@ void Daemon(void)
 		}
 		exit(0);
 	}
-*/
+
 	/* child */
-/*
+
 	if (pid<0) {
 		syslog(LOG_INFO, "fork() failed: %m");
 		unlink(_PATH_VARRUN PID_FILE);
@@ -486,17 +547,17 @@ void Daemon(void)
 	} else {
 		syslog(LOG_INFO, "entering daemon mode");
 	}
-*/
+
 	/* child - Follow the daemon rules in W. Richard Stevens.Advanced
 	   Programming in the UNIX Environment (Addison-Wesley Publishing Co.,
 	   1992). Page 417.). */
-/*
+
 	if (setsid()<0) {
 		syslog(LOG_INFO, "setsid() failed: %m");
 		unlink(_PATH_VARRUN PID_FILE);
 		exit(1);
 	}
-*/
+
 	chdir("/");
 	umask(0);
 
@@ -548,9 +609,10 @@ void Quit(int status)
  */
 void ProcessComandLine(int *argc, char ***argv)
 {
-	int i,quit;
+	int i,rescan,unregister;
 
-	quit = 0;
+	rescan = 0;
+	unregister = 0;
 	for (i=1;i<*argc;i++) {
 		if ((!strcmp((*argv)[i], "-h")) || (!strcmp((*argv)[i], "--help"))) {
 			printf(USAGE);
@@ -564,25 +626,38 @@ void ProcessComandLine(int *argc, char ***argv)
 			quite = 1;
 		} else if ((!strcmp((*argv)[i], "-x")) || (!strcmp((*argv)[i], "--xmessage"))) {
 			xmessage = 1;
-		} else {
+                } else if ((!strcmp((*argv)[i], "-u")) || (!strcmp((*argv)[i], "--unregister"))) {
+                        unregister = 1;
+                } else if ((!strcmp((*argv)[i], "-r")) || (!strcmp((*argv)[i], "--rescan"))) {
+			rescan = 1;
+                } else {
 			fprintf(stderr, "thotswap: unrecognised option %s\n",
 				(*argv)[i]);
 			Quit(1);
 		}
 	}
 
-	/* exit if requested */
+	/* carry out the command line rescaning and removing if requested */
 
-	if (quit>0) {
+	if ((rescan==1) && (unregister==0)) {
+		console = 1;
+		removed = 1;
+		SelectbayLocked();
 		Quit(0);
+	} else if ((rescan==0) && (unregister==1)) {
+		console = 1;
+		SelectbayUnlocked();
+		Quit(0);
+	} else if ((rescan==1) && (unregister==1)) {
+		fprintf(stderr, "thotswap: can't rescan and unregister at the "
+			"same time");
+		Quit(1);
 	}
 
 	return;
 }
 
 
-
-
 /*
  * the entry point of thotswap
  */
@@ -592,7 +667,14 @@ int main(int argc, char *argv[])
 	int v1,v2,v3;
 	FILE *str;
 	struct utsname info;
-	char message[128];
+
+
+	/* this program *must* be run as root */
+
+	if (getuid()) {
+		fprintf(stderr, "thotswap: must be run as root.\n" );
+		return 1;
+	}
 
 	/* do some quick checks on the laptop */
 
@@ -620,11 +702,6 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
-        /* drop root priveleges to minimize the risk of running suid root */
-/*
-        seteuid(getuid());
-        setegid(getgid());
-*/
 	/* check to see if a copy of thotswap is already running */
 
 	if (!access(_PATH_VARRUN PID_FILE, R_OK)) {
@@ -664,9 +741,6 @@ int main(int argc, char *argv[])
 
 	/* enter into daemon mode */
 
-	sprintf(message, "%s", EMERGENCY_ALERT);
-	DisplayMessage(message, "Emergency Battery Alert");
-
 	Daemon();
 
 	return 0;
