@@ -1,6 +1,6 @@
 /* wmtuxtime.c -- WindowMaker dockable applet portion of TuxTime
  *
- * Copyright (c) 1997-99  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
+ * Copyright (c) 1997-2000  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * Based on
  *       wmapm by Chris D. Faulhaber <jedgar@speck.ml.org> and
@@ -8,6 +8,9 @@
  *       mount.app by Steve Borho <borho@stlnet.com>
  *
  * $Log: wmtuxtime.c,v $
+ * Revision 1.4  2000/01/21 22:16:25  jab
+ * fixed bug with blank xmessage windows when launched from dock
+ *
  * Revision 1.3  1999/12/17 12:32:28  jab
  * added lots of machine ID's to stop emails with already known models
  *
@@ -34,7 +37,7 @@
  *
  */
 
-static char const rcsid[]="$Id: wmtuxtime.c,v 1.3 1999/12/17 12:32:28 jab Exp jab $";
+static char const rcsid[]="$Id: wmtuxtime.c,v 1.4 2000/01/21 22:16:25 jab Exp jab $";
 
 
 #include<stdio.h>
@@ -156,7 +159,7 @@ void HandleSignals(void)
  */
 int DisplayMessage(char *message, char *title)
 {
-	int pid,save,lead[2];
+	int pid,lead[2];
 
 	/* create the pipe */
 
@@ -171,14 +174,9 @@ int DisplayMessage(char *message, char *title)
 
 	pid = fork();
 	if (pid>0) {
-		save = dup(1);
-		close(1);
-		dup(lead[1]);
+		write(lead[1], message, strlen(message));
 		close(lead[0]);
 		close(lead[1]);
-		fprintf(stdout, "%s\n", message);
-		close(1);
-		dup(save);
 		return 127;
 	} else if (pid==0) {
 		char *argv[9] = {"xmessage", "-buttons", "Ok", "-title", title,
@@ -883,6 +881,7 @@ int main(int argc, char *argv[])
 	struct passwd *pw;
 	struct stat info;
 
+
 	/* do some quick checks on the laptop */
 
 	if (SciSupportCheck(&version)==1) {
@@ -908,6 +907,7 @@ int main(int argc, char *argv[])
         setegid(getgid());
 
 	switch (id) {
+		case 0xfc09: /* Satellite 2610CDT */
 		case 0xfc0f: /* Satellite 2060CDS/CDT */
 		case 0xfc10: /* Satellite 2550/2590 */
 		case 0xfc11: /* Portage 3110CT */
